"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactComponent = void 0;
const debug_1 = require("debug");
const ts_morph_1 = require("ts-morph");
const TsMorhProject_1 = require("./TsMorhProject");
const log = debug_1.debug("hardhat:plugin:react");
const providersDisallowedInject = ["mainnet"];
class ReactComponent {
    constructor(sourceFile, args, bre, contractContexts, component) {
        this.hre = bre;
        this.args = args;
        this.sourceFile = sourceFile;
        this.contractContexts = contractContexts;
        this.component = component;
    }
    generate() {
        this.useStateStatements();
        this.consoleLog();
        this.getProvider();
        this.getSigner();
        this.getWeb3ModalProvider();
        this.initSideEffect();
        this.contractInits();
        this.handleInitProvider();
        this.renderFunction();
    }
    useStateStatements() {
        this.insertConstStatement("[initializeCounter, setInitializeCounter]", "useState(0)");
        this.insertConstStatement("[currentHardhatProvider, setCurrentHardhatProvider]", `useState("")`);
        this.insertConstStatement("[loading, setLoading]", "useState(false)");
        this.insertConstStatement("[messages, setMessages]", "useState<string[]>([])");
        this.insertConstStatement("[signer, setSigner]", "useState<Signer | undefined>(defaultSigner)");
        this.insertConstStatement("[provider, setProvider]", "useState<providers.Provider | undefined>(defaultProvider)");
        this.insertConstStatement("[currentAddress, setCurrentAddress]", "useState<string>(defaultCurrentAddress)");
        this.insertConstStatement("[fallbackProvider]", `useState<string | undefined>(${this.getFallbackProvider()})`);
        this.insertConstStatement("[providerPriority, setProviderPriority]", `useState<string[]>(${this.toArrayString(this.hre.config.react.providerPriority)})`);
        this.contractContexts.forEach((contract) => {
            this.insertConstStatement(`[${contract.name}, set${contract.name}]`, `useState<${TsMorhProject_1.contractInterfaceName(contract)}>(emptyContract)`);
        });
    }
    getProvider() {
        this.component.addVariableStatement({
            declarationKind: ts_morph_1.VariableDeclarationKind.Const,
            declarations: [
                {
                    name: "getProvider",
                    initializer: (writer) => {
                        writer.write(`async (): Promise<{provider: providers.Provider, hardhatProviderName: string } | undefined> => {
                let hardhatProviderName = "Not set";
                let _providerPriority = [...providerPriority];
                // Fallback provider
                if (fallbackProvider && autoInit && initializeCounter === 0) {
                    if (localStorage.getItem("WEB3_CONNECT_CACHED_PROVIDER") === null) {
                        _providerPriority = _providerPriority.sort((a, b) => {
                            return a === fallbackProvider ? -1 : b === fallbackProvider ? 1 : 0;
                        })
                    }
                }
                  const provider = await _providerPriority.reduce(async (maybeProvider: Promise<providers.Provider | undefined>, providerIdentification)=> {
                    let foundProvider = await maybeProvider
                    if (foundProvider) {
                        return Promise.resolve(foundProvider)
                    }
                    else {
                        switch (providerIdentification.toLowerCase()) {
                            case "web3modal":
                                try {
                                    const provider = await getWeb3ModalProvider()
                                    const web3provider = new ethers.providers.Web3Provider(provider);
                                    hardhatProviderName =  "web3modal";
                                    return Promise.resolve(web3provider)
                                } catch (error) {
                                    return Promise.resolve(undefined)
                                }
                            `);
                        for (let [name, network] of Object.entries(this.hre.config.networks)) {
                            // REVIEW Probably not optimal
                            if (name === "localhost") {
                                name = "hardhat";
                                network = Object.assign(Object.assign({}, network), this.hre.config.networks["hardhat"]);
                            }
                            const isPrioritisedProvider = this.hre.config.react.providerPriority.indexOf(name) !== -1;
                            const hasInject = "inject" in network && network.inject;
                            if (!isPrioritisedProvider) {
                                log("Provider " +
                                    name +
                                    " is not configured as a priortised provider for React. Not adding to provider list.");
                                continue;
                            }
                            let providerConnection = "url" in network ? { url: network.url } : undefined;
                            if (providerConnection) {
                                const urlRegEx = new RegExp("^(?<protocol>.+?//)(?<username>.+?):(?<password>.+?)@(?<address>.+)$");
                                const regExResult = urlRegEx.exec(providerConnection.url);
                                if (regExResult && regExResult.groups) {
                                    if ("username" in regExResult.groups) {
                                        log("Found username " +
                                            regExResult.groups.username +
                                            " in provider url for  " +
                                            name);
                                        providerConnection.user = regExResult.groups.username;
                                    }
                                    if ("password" in regExResult.groups) {
                                        log("Found password " +
                                            regExResult.groups.password +
                                            " in provider url for " +
                                            name);
                                        providerConnection.password = regExResult.groups.password;
                                    }
                                    if ("password" in regExResult.groups &&
                                        "address" in regExResult.groups) {
                                        log("Found protocol and address " +
                                            regExResult.groups.protocol +
                                            regExResult.groups.address +
                                            " in provider url for " +
                                            name);
                                        providerConnection.url =
                                            regExResult.groups.protocol + regExResult.groups.address;
                                    }
                                }
                                if ("providerType" in network) {
                                    providerConnection.providerType = network.providerType;
                                }
                                else {
                                    log("Setting default providerType on " + name);
                                    providerConnection.providerType = "JsonRpcProvider";
                                }
                                writer.write(`case "${name}":
                      try {
                          const provider = new ethers.providers.${providerConnection.providerType}({
                              url: "${providerConnection.url}",`);
                                if (providerConnection.user) {
                                    writer.writeLine(`user: "${providerConnection.user}",`);
                                    if (providerConnection.password) {
                                        writer.writeLine(`password: "${providerConnection.password}"`);
                                    }
                                }
                                writer.write(`});
                  hardhatProviderName =  "${name}";
                  return Promise.resolve(provider)
              } catch (error) {
                  return Promise.resolve(undefined)
              }`);
                            }
                        }
                        writer.write(`default:
                                return Promise.resolve(undefined)
                        }
                    }
                }, Promise.resolve(undefined)) // end reduce
                return provider ? { provider, hardhatProviderName } : undefined
                }`);
                    },
                },
            ],
        });
    }
    getSigner() {
        const writeSigners = (writer) => {
            writer.writeLine(`case "web3modal":
          const web3provider = _provider as ethers.providers.Web3Provider
          return await web3provider.getSigner()`);
            const networksByProviderType = Object.entries(this.hre.config.networks).reduce((acu, [name, network]) => {
                let providerType = "JsonRpcProvider";
                if ("providerType" in network) {
                    if (network.providerType) {
                        providerType = network.providerType;
                    }
                }
                const prev = acu[providerType] ? acu[providerType] : {};
                return Object.assign(Object.assign({}, acu), { [providerType]: Object.assign(Object.assign({}, prev), { [name]: network }) });
            }, {});
            for (const [name, network] of Object.entries(this.hre.config.networks)) {
                writeProviderSigner(writer, name, network);
            }
        };
        const writeProviderSigner = (writer, name, network) => {
            if (providersDisallowedInject.indexOf(name.toLowerCase()) !== -1) {
                log("Provider " + name + " is hardcoded disallowed injected mnemonic.");
                return;
            }
            if ("live" in network) {
                if (network.live) {
                    log("Provider " + name + " is live and therefor disallowing inject.");
                    return;
                }
            }
            const hasInject = "inject" in network && network.inject;
            if (!hasInject && name !== "hardhat") {
                log("Provider " +
                    name +
                    " does not have or have false inject property. Disallowing inject.");
                return;
            }
            if (typeof network.accounts !== "string" &&
                "mnemonic" in network.accounts) {
                const mnemonic = network.accounts.mnemonic;
                log(`Injecting ${name} into React context`);
                log(`Exposing Mnemonic in you React app : ${mnemonic}`);
                writer.writeLine(`case "${name}":
        return ethers.Wallet.fromMnemonic("${mnemonic}").connect(_provider)`);
            }
        };
        this.component.addVariableStatement({
            declarationKind: ts_morph_1.VariableDeclarationKind.Const,
            declarations: [
                {
                    name: "getSigner",
                    initializer: (writer) => {
                        writer.write(`async ( _provider: providers.Provider, hardhatProviderName: string): Promise<Signer | undefined> => {
                switch (hardhatProviderName) {`);
                        writeSigners(writer);
                        writer.write(`default:
                        return undefined
                }
            }`);
                    },
                },
            ],
        });
    }
    getWeb3ModalProvider() {
        const providerOptions = {
            walletconnect: {
                package: `${"WalletConnectProvider"}`,
                options: {
                    infuraId: "INFURA_ID",
                },
            },
        };
        this.component.addVariableStatement({
            declarationKind: ts_morph_1.VariableDeclarationKind.Const,
            declarations: [
                {
                    name: "getWeb3ModalProvider",
                    initializer: async (writer) => {
                        writer.write(`async (): Promise<any> => {
                const providerOptions: IProviderOptions = {
                  `);
                        if (this.hre.config.react.providerOptions) {
                            for (const [providerName, providerOptions] of Object.entries(this.hre.config.react.providerOptions)) {
                                if (providerName === "walletconnect") {
                                    if ("infuraId" in providerOptions.options) {
                                        writer.write(`walletconnect: {
                      package: WalletConnectProvider,
                      options: {
                        infuraId: "${providerOptions.options.infuraId}"
                      }
                    }`);
                                    }
                                    else if ("rpc" in providerOptions.options) {
                                        writer.write(`walletconnect: {
                      package: WalletConnectProvider,
                      options: {
                        rpc: {`);
                                        for (const [chainId, providerUrl] of Object.entries(providerOptions.options.rpc)) {
                                            writer.writeLine(`${chainId}: "${providerUrl}"`);
                                        }
                                        writer.write(`}
                      }
                    }`);
                                    }
                                    else {
                                        throw Error("Found WalletConnect web3modal configuration, but no infuraId in options. Please add react.providerOptions.walletconnect.options.infuraId in your Hardhat.config ");
                                    }
                                }
                            }
                        }
                        writer.write(`
                };
                const web3Modal = new Web3Modal({
                    // network: "mainnet",
                    cacheProvider: false,
                    providerOptions, // required
                });
                return await web3Modal.connect();
            }`);
                    },
                },
            ],
        });
    }
    initSideEffect() {
        this.component.addStatements((writer) => {
            writer.write(`
      useEffect(() => {
          let subscribed = true
          const doAsync = async () => {
            const finish = (text: string) => {
              setLoading(false)
              setMessages(old => [...old, text])
            }
            const finishWithContracts = (text: string) => {`);
            this.contractContexts.forEach((contract) => {
                writer.writeLine(`set${contract.name}(get${contract.name}(_provider, _signer))`);
            });
            writer.write(`finish(text)
            }
            if (!autoInit && initializeCounter === 0) return finish("Auto init turned off.")
            setLoading(true)
            setMessages(old => [...old, "Initiating Symfoni React"])
            const providerObject = await getProvider() // getProvider can actually return undefined, see issue https://github.com/microsoft/TypeScript/issues/11094

            if (!subscribed || !providerObject) return finish("No provider or signer.")
            const _provider = providerObject.provider
            setProvider(_provider)
            setMessages(old => [...old, "Useing " + providerObject.hardhatProviderName])
            setCurrentHardhatProvider(providerObject.hardhatProviderName)
            const _signer = await getSigner(_provider, providerObject.hardhatProviderName);

            if (!subscribed || !_signer) return finishWithContracts("Provider, without signer.")
            setSigner(_signer)
            setMessages(old => [...old, "Useing signer"])
            const address = await _signer.getAddress()

            if (!subscribed || !address) return finishWithContracts("Provider and signer, without address.")
            setCurrentAddress(address)
            `);
            writer.write(`
      return finishWithContracts("Completed Symfoni context initialization.")
        };
        doAsync();
        return () => { subscribed = false }
      }, [initializeCounter])`);
        });
    }
    consoleLog() {
        this.component.addStatements(`useEffect(() => {
        if(messages.length > 0)
          console.debug(messages.pop())
    }, [messages])`);
    }
    contractInits() {
        this.contractContexts.forEach((contract) => {
            this.component.addVariableStatement({
                declarationKind: ts_morph_1.VariableDeclarationKind.Const,
                declarations: [
                    {
                        name: `get${contract.name}`,
                        initializer: async (writer) => {
                            writer.writeLine(`(_provider: providers.Provider, _signer?: Signer ) => {`);
                            if (contract.contractAddress) {
                                writer.write(`
                  const contractAddress = "${contract.contractAddress}"
                  const instance = _signer ? ${contract.typechainFactoryName}.connect(contractAddress, _signer) : ${contract.typechainFactoryName}.connect(contractAddress, _provider)
                `);
                            }
                            else {
                                writer.writeLine(`let instance = _signer ? ${contract.typechainFactoryName}.connect(ethers.constants.AddressZero, _signer) : ${contract.typechainFactoryName}.connect(ethers.constants.AddressZero, _provider)`);
                            }
                            writer.write(`const contract: ${TsMorhProject_1.contractInterfaceName(contract)} = {
                  instance: instance  ,
                  factory: _signer ? new ${contract.typechainFactoryName}(_signer) : undefined,
                } 
                return contract`);
                            writer.write(`}
              `);
                        },
                    },
                ],
            });
        });
    }
    handleInitProvider() {
        this.component.addStatements(`
      const handleInitProvider = (provider?: string) => {
        if (provider) {
            setProviderPriority(old => old.sort((a, b) => {
                return a === provider ? -1 : b === provider ? 1 : 0;
            }))
        }
        setInitializeCounter(initializeCounter + 1) 
      }
    `);
    }
    renderFunction() {
        const body = (writer) => {
            writer.writeLine(`{showLoading && loading ?
          props.loadingComponent
              ? props.loadingComponent
              : <div>
                  {messages.map((msg, i) => (
                      <p key={i}>{msg}</p>
                  ))}
              </div>
          : props.children
      }`);
        };
        const openContext = (writer) => {
            this.contractContexts.forEach((contract) => {
                writer.writeLine(`<${contract.name}Context.Provider value={${contract.name}}>`);
            });
        };
        const closeContext = (writer) => {
            this.contractContexts.reverse().forEach((contract) => {
                writer.writeLine(`</${contract.name}Context.Provider >`);
            });
        };
        this.component.addStatements((writer) => {
            writer.write(`return (
          <SymfoniContext.Provider value={{ init: (provider) => handleInitProvider(provider), providers: providerPriority,currentHardhatProvider, loading, messages }}>
            <ProviderContext.Provider value={[provider, setProvider]}>
                <SignerContext.Provider value={[signer, setSigner]}>
                    <CurrentAddressContext.Provider value={[currentAddress, setCurrentAddress]}>`);
            openContext(writer);
            body(writer);
            closeContext(writer);
            writer.write(`           </CurrentAddressContext.Provider>
                </SignerContext.Provider>
            </ProviderContext.Provider>
          </SymfoniContext.Provider>
        )`);
        });
    }
    // Helpers
    insertConstStatement(name, initializer) {
        this.component.addVariableStatements([
            {
                declarationKind: ts_morph_1.VariableDeclarationKind.Const,
                declarations: [
                    {
                        name,
                        initializer,
                    },
                ],
            },
        ]);
    }
    toArrayString(arr) {
        return "[" + arr.map((i) => `"` + i + `"`).join(",") + "]";
    }
    getFallbackProvider() {
        if (this.hre.config.react.fallbackProvider) {
            return `"${this.hre.config.react.fallbackProvider}"`;
        }
        return undefined;
    }
}
exports.ReactComponent = ReactComponent;
//# sourceMappingURL=ReactComponent.js.map